(*
    ===========================================================================

    iFont.pas

    ---------------------------------------------------------------------------

    2003/12/02  쐬
    2004/01/04  C

    ===========================================================================
*)
unit iFont;

interface

uses
    Windows, Messages, Forms, SysUtils, Classes, Contnrs, iFiles, IniFiles;

(*
    =================================================
    t@C
    =================================================
*)
type
    TFileInfo = record
        CreateDate: TDateTime;
        ModifyDate: TDateTime;
        AccessDate: TDateTime;
        FileSize  : integer;
    end;

(*
    =================================================
    pNX\bh^
    =================================================
*)
type
    TLiveMsgProc = procedure(const AMsg: string) of object;

(*
    =================================================
    
    =================================================
*)
type
    TPostIt = (
                piNone    = -1
              , piWhite   = 0
              , piGray    = 1
              , piRed     = 2
              , piYellow  = 3
              , piGreen   = 4
              , piCyan    = 5
              , piBlue    = 6
              , piMagenda = 7
            );

(*
    =================================================
    ꎞItHgCXg[NX
    =================================================
*)
type
    TFontFile = class
    private
        FFullPath   : string;
        FInstalled  : Boolean;
        FAddedCount : integer;
        FAddedFont  : string;
        FAddedFonts : string;
        FFileInfo   : TFileInfo;
        FPostIt     : TPostIt;
        procedure   SetFullPath(AValue: string);
        function    GetFileName(): string;
        function    GetFileHint(): string;
        function    GetDispName(): string;
    public
        FLiveMsgProc: TLiveMsgProc; 
        function    Install(): Boolean;
        procedure   UnInstall();
        procedure   OpenAsIs();
        procedure   OpenProperty();
        procedure   OpenParentDir();
        procedure   TrashShoot();
        constructor Create(AFullPath: string);
        destructor  Destroy(); override;
        function    GetAddedFontName(const Default: string): string;
    protected

    published
        property    FileName: string read GetFileName;
        property    FullPath: string read FFullPath write SetFullPath;
        property    Installed: Boolean read FInstalled;
        property    AddedCount: integer read FAddedCount;
        property    AddedFont: string read FAddedFont;
        property    AddedFonts: string read FAddedFonts;
        property    FileHint: string read GetFileHint;
        property    PostIt: TPostIt read FPostIt write FPostIt;
        property    DispName: string read GetDispName;
    end;

(*
    =================================================
    ꎞItHgCXg[NXpXg
    =================================================
*)
type
    TFontFileSortType = (ffstName, ffstPath, ffstMark, ffstInstalled);

type
    TFontFileList = class(TObjectList)
    private
        FLiveMsgProc: TLiveMsgProc;
        function    Exists(AFontName: string): Boolean;
        procedure   SetLiveMsgProc(AValue: TLiveMsgProc);
    public
        constructor Create();
        destructor  Destroy(); override;
        function    AddFontFile(AFullPath: string): TFontFile;
        function    GetItem(AIndex: integer): TFontFile;

        procedure   InstallByMark(APostIt: TPostIt);
        procedure   UnInstallByMark(APostIt: TPostIt);
        procedure   DeleteByMark(APostIt: TPostIt);
        procedure   TrashByMark(APostIt: TPostIt);

        procedure   SortBy(AType: TFontFileSortType);

        procedure   SaveToFile(AFileName: string);
        procedure   LoadFromFile(AFileName: string);
    protected

    published
        property    LiveMsgProc: TLiveMsgProc write SetLiveMsgProc;
    end;

(*
    =================================================
    ΉĂgq
    =================================================
*)
const
	FONT_EXT = '.fon,.fnt,.ttf,.ttc,.fot,.otf,.mmm,.pfb,.pfm';

(*
    =================================================
    [eBeB֐
    =================================================
*)
procedure   GetScreenFonts(pResult: TStrings);

function    KanaToChars(AKana: string): string;

const
    AIUEO_TABLE: array[0..4, 0..9] of Char =
    (
        ('0', 'o', '7', 'j', 'f', 'u', 'q', 'x', 't', '3'),
        ('}', 'l', '[', 'n', 'v', 'i', 'a', 'd', 'g', 'e'),
        ('y', '.', '8', ']', '2', '1', 'z', 'r', 'h', '4'),
        ('[', ';', '[', '/', '^', ',', 'w', 'p', ':', '5'),
        ('[', 'N', '9', 'm', '-', 'k', 's', 'c', 'b', '6')
    );

implementation

var
    WorkSL1: TStringList = nil;
    WorkSL2: TStringList = nil;

(*
    =================================================
    [eBeB֐
    =================================================
*)

(*
    -------------------------------------------------
    t@CtEt@CTCY(R[h)
    -------------------------------------------------
    http://hp.vector.co.jp/authors/VA009712/take/delphi/
                               kabesys.htm#datetimefile2
    http://forum.nifty.com/fdelphi/faq/00048.htm
    -------------------------------------------------
*)
function    GetFileInfo(const AFullPath: string): TFileInfo;
var
    hFile: integer;
    ftCreateDate, ftModifyDate, ftAccessDate: TFileTime;
    ltCreateDate, ltModifyDate, ltAccessDate: TFileTime;
    stCreateDate, stModifyDate, stAccessDate: TSystemTime;
begin
    //  
    FillChar(Result, SizeOf(Result), #0);
    //  ǂ݂ƂpŃt@CJ
    hFile := FileOpen(AFullPath, fmOpenRead);
    try
        //  t@C̓擾
        GetFileTime(hFile, @ftCreateDate, @ftAccessDate, @ftModifyDate);
        //  [JɕύX
        FileTimeToLocalFileTime(ftCreateDate, ltCreateDate);
        FileTimeToLocalFileTime(ftModifyDate, ltModifyDate);
        FileTimeToLocalFileTime(ftAccessDate, ltAccessDate);
        //  VXeɕύX
        FileTimeToSystemTime(ltCreateDate, stCreateDate);
        FileTimeToSystemTime(ltModifyDate, stModifyDate);
        FileTimeToSystemTime(ltAccessDate, stAccessDate);
        //  쐬
        Result.CreateDate := SystemTimeToDateTime(stCreateDate);
        //  C
        Result.ModifyDate := SystemTimeToDateTime(stModifyDate);
        //  ANZX
        Result.AccessDate := SystemTimeToDateTime(stAccessDate);
        //  t@CTCY
        Result.FileSize := FileSeek(hFile, 0, 2);
    finally
        //  
        FileClose(hFile);
    end;
end;

(*
    -------------------------------------------------
    t@CtEt@CTCY()
    -------------------------------------------------
*)
function    GetFileInfoStr(const AFileInfo: TFileInfo): string;
const
    DAT_STR = 'yyyy/mm/dd hh:nn:ss';
    FMT_STR = '쐬:%s'#13#10'XV:%s'#13#10'ڐG:%s'#13#10'TCY:%dbytes';
begin
    with AFileInfo do
    begin
        Result := Format(FMT_STR, [
                        FormatDateTime(DAT_STR, CreateDate)
                        , FormatDateTime(DAT_STR, ModifyDate)
                        , FormatDateTime(DAT_STR, AccessDate)
                        , FileSize]);
    end;
end;

(*
    -------------------------------------------------
    2 oCg̍ŏ̃oCg
    -------------------------------------------------
*)
function IsLeadByte(AStr: string; AByteIndex: integer): Boolean;
begin
    Result := (StrByteType(PChar(AStr), AByteIndex) = mbLeadByte);
end;

(*
    -------------------------------------------------
    gq`FbN
    -------------------------------------------------
*)
function    HasExtension(APath, AExtCommaText: string): Boolean;
var
    EXT: string;
begin
    EXT := UpperCase(ExtractFileExt(APath));
    Result := (Pos(EXT, UpperCase(AExtCommaText)) > 0);
end;

(*
    =================================================
    XN[tHg̈ꗗ\B
    =================================================
*)
procedure   GetScreenFonts(pResult: TStrings);
var
    i: integer;
begin
    //ScreenϐoB
    pResult.Assign(Screen.Fonts);
    //cp(?)tHgĈłǂB
    for i := pResult.Count - 1 downto 0 do
    begin
        if Copy(pResult[i], 1, 1) = '@' then
        begin
            pResult.Delete(i);
        end;
    end;
end;

(*
    =====================================================================
    TFontFile
    =====================================================================
*)

(*
    -------------------------------------------------
    t@C̕ύX
    -------------------------------------------------
*)
procedure   TFontFile.SetFullPath(AValue: string);
begin
    //  CXg[ĂȂACXg[
    if FInstalled then UnInstall();
    //  ݒ肵Ȃ
    Create(AValue);
end;

(*
    -------------------------------------------------
    t@C(̂)擾
    -------------------------------------------------
*)
function    TFontFile.GetFileName(): string;
begin
    Result := ExtractFileName(FFullPath);
end;

(*
    -------------------------------------------------
    qg擾
    -------------------------------------------------
*)
function    TFontFile.GetFileHint(): string;
const
    FMT_STR = 'FONT:%s'#13#10'%s'#13#10'ǉ:%d'#13#10'ǉ:%s';
begin
    Result := Format(FMT_STR, [FFullPath
                               , GetFileInfoStr(FFileInfo)
                               , FAddedCount
                               , FAddedFonts]
                    );
end;

(*
    -------------------------------------------------
    \p
    -------------------------------------------------
*)
function    TFontFile.GetDispName(): string;
begin
    if FInstalled then Result := '' + FileName
                  else Result := '' + FileName;
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    RXgN^
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
constructor TFontFile.Create(AFullPath: string);
begin
    inherited Create;
    FLiveMsgProc := nil;
    FFullPath := AFullPath;
    FInstalled := False;
    FAddedFonts := '';
    FFileInfo := GetFileInfo(FFullPath);
    FPostIt := piNone;
    FAddedCount := 0;
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    fXgN^
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
destructor  TFontFile.Destroy();
begin
    FLiveMsgProc := nil;
    //  CXg[ĂȂACXg[
    if FInstalled then UnInstall();
    inherited Destroy;
end;

(*
    =================================================
    [2004/03/04]StringList̍𒲂ׂ
    -------------------------------------------------
    AListɂASubListɂȂ̂񋓂
    =================================================
*)
function    DiffStringLists(ASubList, AList: TStringList): string;
var
    i, index: integer;
    Temp: string;
begin
    Result := '';
    ASubList.Sort();
    for i := 0 to AList.Count - 1 do
    begin
        Temp := AList[i];
        if ASubList.Find(Temp, index) then continue;
        Result := Result + ',' + Temp;     
    end;
    Delete(Result, 1, 1);
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    CXg[
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
function    TFontFile.Install(): Boolean;
var
    Msg: string;
begin
    if not FInstalled then
    begin
        //  ΉtHg̃NA
        FAddedFont := '';
        FAddedFonts := '';
        //  ݂̎gp\tHg̎擾
        GetScreenFonts(WorkSL1);
        //  CXg[
        FAddedCount := AddFontResource(PChar(FFullPath));
        FInstalled := (FAddedCount <> 0);
        //  
        if FInstalled then
        begin
            //  ̃AvP[Vɂʒm
            SendMessage(HWND_BROADCAST, WM_FONTCHANGE, 0, 0);
            //  ǉ̎gp\tHg̎擾
            GetScreenFonts(WorkSL2);
            //  gp\tHg̑`FbN
            FAddedFonts := DiffStringLists(WorkSL1, WorkSL2);
            //  ̂1oĂ
            WorkSL1.CommaText := FAddedFonts;
            if (WorkSL1.Count > 0) then FAddedFont := WorkSL1[0];
            //  p
            if Assigned(FLiveMsgProc) then
            begin
                Msg := Format('[  OK  ]INSTALL %s/%s', [FileName, FAddedFonts]);
                FLiveMsgProc(Msg);
            end;
        end
        else
        begin
            //  p
            if Assigned(FLiveMsgProc) then
            begin
                Msg := Format('[FAILED]INSTALL %s', [FileName]);
                FLiveMsgProc(Msg);
            end;
        end;
    end
    else
    begin
        //  p
        if Assigned(FLiveMsgProc) then
        begin
            Msg := Format('[ SKIP ]ALREADY INSTALLED %s', [FileName]);
            FLiveMsgProc(Msg);
        end;
    end;
    Result := FInstalled;
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    ACXg[
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
procedure   TFontFile.UnInstall();
var
    Msg: string;
begin
    if FInstalled then
    begin
        //  폜
        RemoveFontResource(PChar(FFullPath));
        //  ̃AvP[Vɂʒm
        SendMessage(HWND_BROADCAST, WM_FONTCHANGE, 0, 0);
        //  p
        if Assigned(FLiveMsgProc) then
        begin
            Msg := Format('[  OK  ]UNINSTALL %s', [FileName]);
            FLiveMsgProc(Msg);
        end;
        FInstalled := False;
    end
    else
    begin
        //  p
        if Assigned(FLiveMsgProc) then
        begin
            Msg := Format('[ SKIP ]NOT INSTALLED %s', [FileName]);
            FLiveMsgProc(Msg);
        end;
    end;
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    VFł̃_uNbN̂悤ɊJ
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
procedure   TFontFile.OpenAsIs();
begin
    SimpleShellExecute(FFullPath);
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    VFł̃t@CvpeB_CAOJ
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
procedure   TFontFile.OpenProperty();
begin
    OpenFilePropertyDialog(FFullPath);
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    ̃t@C܂ĂtH_J
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
procedure   TFontFile.OpenParentDir();
begin
    SimpleShellExecute(ExtractFilePath(FFullPath));
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    ̃t@C܂ĂtH_J
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
procedure   TFontFile.TrashShoot();
begin
    UnInstall();
    SendToRecycleBin(FFullPath);
end;

(*
    =================================================
    [2004/03/04]ǉꂽtHg̖OSɎ擾
    =================================================
*)
function    TFontFile.GetAddedFontName(const Default: string): string;
begin
    Result := Default;
    if (FAddedFont <> '') and (FInstalled) then Result := FAddedFont;
end;

(*
    =====================================================================
    TFontFileList
    =====================================================================
*)

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    RXgN^
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
constructor TFontFileList.Create();
begin
    inherited Create;
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    fXgN^
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
destructor  TFontFileList.Destroy();
begin
    inherited Destroy;
end;

(*
    -------------------------------------------------
    o^ς݃`FbN
    -------------------------------------------------
*)
function    TFontFileList.Exists(AFontName: string): Boolean;
var
    i: integer;
    F1, F2: string;
begin
    Result := true;
    //  ׂt@C
    F2 := UpperCase(AFontName);
    //  ̃t@CXgɂ܂?
    for i := 0 to Self.Count - 1 do
    begin
        F1 := UpperCase(GetItem(i).FileName);
        if (F1 = F2) then Exit;
    end;
    Result := false;
end;

(*
    -------------------------------------------------
    p֐Zbg
    -------------------------------------------------
*)
procedure   TFontFileList.SetLiveMsgProc(AValue: TLiveMsgProc);
var
    i: integer;
    Item: TFontFile;
begin
    FLiveMsgProc := AValue;
    for i := Self.Count - 1 downto 0 do
    begin
        Item := Self.GetItem(i);
        Item.FLiveMsgProc := Self.FLiveMsgProc;
    end;
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    tHgt@Cǉ
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
function    TFontFileList.AddFontFile(AFullPath: string): TFontFile;
var
    Item: TFontFile;
begin
    Result := nil;
    //  ܂ǉĂȂt@CȂǉ
    if not Exists(ExtractFileName(AFullPath)) then
    begin
        Item := TFontFile.Create(AFullPath);
        Item.FLiveMsgProc := Self.FLiveMsgProc;
        Self.Add(Item);
        Result := Item;
    end;
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    LXgς݃IuWFNgo
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
function    TFontFileList.GetItem(AIndex: integer): TFontFile;
begin
    Result := TFontFile(Self.Items[AIndex]);
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    w}[NCXg[
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
procedure   TFontFileList.InstallByMark(APostIt: TPostIt);
var
    i, Counter: integer;
    Item: TFontFile;
    Msg: string;
begin
    Counter := 0;
    for i := 0 to Self.Count - 1 do
    begin
        Item := Self.GetItem(i);
        if (Item.PostIt = APostIt) then
        begin
            if Item.Install() then Inc(Counter);
        end;
    end;
    //  p
    if Assigned(FLiveMsgProc) then
    begin
        Msg := Format('[  OK  ]INSTALL ABOUT %d FONTS', [Counter]);
        FLiveMsgProc(Msg);
    end;
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    w}[NACXg[
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
procedure   TFontFileList.UnInstallByMark(APostIt: TPostIt);
var
    i, Counter: integer;
    Item: TFontFile;
    Msg: string;
begin
    Counter := 0;
    for i := 0 to Self.Count - 1 do
    begin
        Item := Self.GetItem(i);
        if (Item.PostIt = APostIt) then
        begin
            Item.UnInstall();
            Inc(Counter);
        end;
    end;
    //  p
    if Assigned(FLiveMsgProc) then
    begin
        Msg := Format('[  OK  ]UNINSTALL ABOUT %d FONTS', [Counter]);
        FLiveMsgProc(Msg);
    end;
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    w}[NXgNA
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
procedure   TFontFileList.DeleteByMark(APostIt: TPostIt);
var
    i, Counter: integer;
    Item: TFontFile;
    Msg: string;
begin
    Counter := 0;
    for i := Self.Count - 1 downto 0 do
    begin
        Item := Self.GetItem(i);
        if (Item.PostIt = APostIt) then
        begin
            Self.Delete(i);
            Inc(Counter);
        end;
    end;
    //  p
    if Assigned(FLiveMsgProc) then
    begin
        Msg := Format('[  OK  ]DELETE FROM LIST %d FONTS', [Counter]);
        FLiveMsgProc(Msg);
    end;
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    w}[NݔV[g
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)
procedure   TFontFileList.TrashByMark(APostIt: TPostIt);
var
    i, Counter: integer;
    Item: TFontFile;
    Msg: string;
begin
    Counter := 0;
    for i := Self.Count - 1 downto 0 do
    begin
        Item := Self.GetItem(i);
        if (Item.PostIt = APostIt) then
        begin
            Item.TrashShoot();
            Self.Delete(i);
            Inc(Counter);
        end;
    end;
    //  p
    if Assigned(FLiveMsgProc) then
    begin
        Msg := Format('[  OK  ]SHOOT TRASH %d FONTS', [Counter]);
        FLiveMsgProc(Msg);
    end;
end;

(*
    +++++++++++++++++++++++++++++++++++++++++++++++++
    \[g
    +++++++++++++++++++++++++++++++++++++++++++++++++
*)

    (* t@C\[g *)
    function    _SortByName(Item1, Item2: Pointer): Integer;
    var
        Temp1, Temp2: string;
    begin
        Temp1 := TFontFile(Item1).FileName;
        Temp2 := TFontFile(Item2).FileName;
        result := AnsiCompareFileName(Temp1, Temp2);
    end;

    (* tpX\[g *)
    function    _SortByPath(Item1, Item2: Pointer): Integer;
    var
        Temp1, Temp2: string;
    begin
        Temp1 := TFontFile(Item1).FullPath;
        Temp2 := TFontFile(Item2).FullPath;
        result := AnsiCompareFileName(Temp1, Temp2);
    end;

    (* }[N\[g *)
    function    _SortByMark(Item1, Item2: Pointer): Integer;
    var
        Temp1, Temp2: integer;
    begin
        Temp1 := Ord(TFontFile(Item1).PostIt);
        Temp2 := Ord(TFontFile(Item2).PostIt);
//        result := (Temp1 - Temp2);
        result := (Temp2 - Temp1);
        if (result = 0) then result := _SortByName(Item1, Item2);         
    end;

    (* ԃ\[g *)
    function    _SortByInstalled(Item1, Item2: Pointer): Integer;
    var
        Temp1, Temp2: Integer;
    begin
        Temp1 := Integer(TFontFile(Item1).Installed);
        Temp2 := Integer(TFontFile(Item2).Installed);
//        result := (Temp1 - Temp2);    //  CXg[̂̂
        result := (Temp2 - Temp1);
        if (result = 0) then result := _SortByName(Item1, Item2);
    end;
    

procedure   TFontFileList.SortBy(AType: TFontFileSortType);
begin
    case AType of
    ffstName     : Self.Sort(_SortByName);
    ffstPath     : Self.Sort(_SortByPath);
    ffstMark     : Self.Sort(_SortByMark);
    ffstInstalled: Self.Sort(_SortByInstalled);
    end;
end;

(*
    =================================================================
    [2004/03/02]t@Cɕۑ
    =================================================================
*)
const DELIM_CHAR = #9;

procedure   TFontFileList.SaveToFile(AFileName: string);
var
    i: integer;
    Saver: TStrings;
    Item: TFontFile;
    Temp: string;
begin
    Saver := TStringList.Create;
    try
        for i := 0 to Self.Count - 1 do
        begin
            Item := Self.GetItem(i);
            Temp := Format('%d' + DELIM_CHAR + '%s', [Ord(Item.PostIt), Item.FullPath]);
            Saver.Add(Temp);
        end;
        Saver.SaveToFile(AFileName);
    finally
        Saver.Free;
    end;
end;

(*
    =================================================================
    [2004/03/02]t@CǍ
    =================================================================
*)
procedure   TFontFileList.LoadFromFile(AFileName: string);
var
    i, DelimPos: integer;
    Loader: TStrings;
    Line, Path: string;
    MarkIndex: integer;
    FontFile: TFontFile;
begin
    if not FileExists(AFileName) then exit;
    Loader := TStringList.Create;
    try
        //  ǂݍ
        Loader.LoadFromFile(AFileName);
        for i := 0 to Loader.Count - 1 do
        begin
            //  esɑ΂
            Line := Loader[i];
            //  f~^ʒu擾
            DelimPos := Pos(DELIM_CHAR, Line);
            //  Ȃ΃XLbv
            if (DelimPos = 0) then continue;
            //  Oɕ
            MarkIndex := StrToIntDef(Copy(Line, 1, DelimPos - 1), Ord(piNone));
            Path := Copy(Line, DelimPos + 1, Length(Line));
            //  t@CpX̐`FbN
            if not FileExists(Path) then continue;
            //  gq`FbN
            if (Pos(UpperCase(ExtractFileExt(Path)), UpperCase(FONT_EXT)) = 0) then continue;
            //  ǉ
            FontFile := Self.AddFontFile(Path);
            //  ǉł
            if (FontFile = nil) then continue;
            //  }[LO
            if (MarkIndex < Ord(Low(TPostIt))) then
            begin
                MarkIndex := Ord(Low(TPostIt));
            end
            else
            if (MarkIndex > Ord(High(TPostIt))) then
            begin
                MarkIndex := Ord(High(TPostIt));
            end;
            FontFile.PostIt := TPostIt(MarkIndex);
        end;
    finally
        Loader.Free;
    end;
end;





















var
    KanaTable: TStrings;

(*
    =================================================
    JitHg̃JiƉpƂ̑Ήt
    =================================================
*)
function    KanaToChar(AKana: string): Char;
var
    Index: integer;
begin
    Index := KanaTable.IndexOf(AKana);
    if Index <> -1 then
    begin
        Result := Chr(Integer(KanaTable.Objects[Index]))
    end
    else
    begin
        Result := ' ';
    end;
end;

(*
    =================================================
    JitHg̃JiBƉpBƂ̑Ήt
    =================================================
*)
function    KanaToChars(AKana: string): string;
var
    i: integer;
begin
    Result := '';
    i := 1;
    while i <= Length(AKana) do
    begin
        if IsLeadByte(AKana[i], 0) then
        begin
            Result := Result + KanaToChar(AKana[i] + AKana[i + 1]);
            Inc(i, 2);
        end
        else
        begin
            Inc(i, 1);
        end;
    end;
end;

(*
    =================================================
    JitHg̃JiƉpƂ̑Ήte[u
    =================================================
*)
procedure   SetKanaTable();
begin
    with KanaTable do
    begin
        Clear;
        AddObject('', TObject('3'));
        AddObject('', TObject('e'));
        AddObject('', TObject('4'));
        AddObject('', TObject('5'));
        AddObject('', TObject('6'));

        AddObject('', TObject('t'));
        AddObject('', TObject('g'));
        AddObject('', TObject('h'));
        AddObject('', TObject(':'));
        AddObject('', TObject('b'));

        AddObject('', TObject('x'));
        AddObject('', TObject('d'));
        AddObject('', TObject('r'));
        AddObject('', TObject('p'));
        AddObject('', TObject('c'));

        AddObject('', TObject('q'));
        AddObject('', TObject('a'));
        AddObject('', TObject('z'));
        AddObject('', TObject('w'));
        AddObject('', TObject('s'));

        AddObject('', TObject('u'));
        AddObject('', TObject('i'));
        AddObject('', TObject('1'));
        AddObject('', TObject(','));
        AddObject('', TObject('k'));

        AddObject('', TObject('f'));
        AddObject('', TObject('v'));
        AddObject('', TObject('2'));
        AddObject('', TObject('^'));
        AddObject('', TObject('-'));

        AddObject('', TObject('j'));
        AddObject('', TObject('n'));
        AddObject('', TObject(']'));
        AddObject('', TObject('/'));
        AddObject('', TObject('m'));

        AddObject('', TObject('7'));
        AddObject('', TObject('8'));
        AddObject('', TObject('9'));

        AddObject('', TObject('o'));
        AddObject('', TObject('l'));
        AddObject('', TObject('.'));
        AddObject('', TObject(';'));
        AddObject('', TObject('N'));

        AddObject('', TObject('0'));
        AddObject('', TObject('}'));
        AddObject('', TObject('y'));

        //

        AddObject('', TObject('J'));
        AddObject('', TObject('G'));
        AddObject('', TObject('H'));
        AddObject('', TObject('*'));
        AddObject('', TObject('B'));

        AddObject('', TObject('X'));
        AddObject('', TObject('D'));
        AddObject('', TObject('R'));
        AddObject('', TObject('P'));
        AddObject('', TObject('C'));

        AddObject('', TObject('Q'));
        AddObject('', TObject('A'));
        AddObject('', TObject('M'));
        AddObject('', TObject('W'));
        AddObject('', TObject('S'));

        AddObject('', TObject('F'));
        AddObject('', TObject('V'));
        AddObject('', TObject('?'));
        AddObject('', TObject('`'));
        AddObject('', TObject('='));

        AddObject('', TObject('U'));
        AddObject('', TObject('I'));
        AddObject('', TObject('!'));
        AddObject('', TObject('<'));
        AddObject('', TObject('K'));

        AddObject('', TObject('#'));
        AddObject('', TObject('E'));
        AddObject('', TObject('$'));
        AddObject('', TObject('%'));
        AddObject('', TObject('&'));

        AddObject('', TObject('('));
        AddObject('', TObject(')'));
        AddObject('', TObject('Z'));
        AddObject('|', TObject('['));
    end;

end;

(*
    =================================================
    jbg
    =================================================
*)
initialization

    //  ƗpXg쐬
    if (WorkSL1 = nil) then WorkSL1 := THashedStringList.Create;
    if (WorkSL2 = nil) then WorkSL2 := THashedStringList.Create;

    KanaTable := TStringList.Create;
    SetKanaTable();

finalization

    //  ƗpXgj
    WorkSL1.Free;
    WorkSL1 := nil;
    WorkSL2.Free;
    WorkSL2 := nil;

    KanaTable.Free;

end.

